#! /usr/bin/env python

#	Copyright 2013 by Reach Technology, Inc.
#	All Rights Reserved

from gimpfu import *
import slcd_common

ORNT_HORIZ = 1
ORNT_VERT = 0


#
# Adds a bitmap to an existing image.  Bitmap is loaded from the specified
# file, into a new layer.  The SLCD command to draw the bitmap is used as
# the layer name, with the token COORDS used as a placeholder for the x/y
# coordinates.
#

def Add_Bitmap(
			inImage,
			inBitmapFilename,
#			inCentered,
			inNickName
):
	"""Add a bitmap to an existing screen image."""
	if not slcd_common.user_item_is_file(inBitmapFilename):
		fail("Invalid bitmap file:\n" + inBitmapFilename)

	try:
		# DRAW BITMAP syntax:
		#		xi <index> x y
		#	or, if inCentered is true,
		#		xim <index> x y
		#	"COORDS" is used for "x y", and will be replaced with actual x and y values later
# 'centered' NYI, and may not be needed at all
		#		if inCentered:
#			command = "xim"
#		else:
#			command = "xi"
		command = "xi"
		bmp = slcd_common.get_bitmap_index(inBitmapFilename)
		commandLine =	command + \
						" " + str(bmp) + \
						" COORDS" + \
						" // @" + str(inNickName)	# nickname is added as a comment
		slcd_common.do_bitmap_add(inImage, commandLine, inBitmapFilename)
		slcd_common.set_user_item(slcd_common.bitmap(), inBitmapFilename)
	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return

	
def Add_Windowed_Bitmap(
			inImage,
			inBitmapFilename,
			inOrient,
			inLength,
			inOffset,
			inNickName
):
	"""Add a bitmap to an existing screen image."""
	if not slcd_common.user_item_is_file(inBitmapFilename):
		fail("Invalid bitmap file:\n" + inBitmapFilename)

	try:
		# DRAW WINDOWED BITMAP syntax:
		#		xio <index> <x> <y> <0|1> <length> <offset>
		#	"COORDS" is used for "x y", and will be replaced with actual x and y values later
		command = "xio"
		bmp = slcd_common.get_bitmap_index(inBitmapFilename)
		commandLine =	command + \
						" " + str(bmp) + \
						" COORDS" + \
						" " + str(inOrient) + \
						" " + str(inLength) + \
						" " + str(inOffset) + \
						" // @" + str(inNickName)	# nickname is added as a comment
		bitmapLayer = slcd_common.do_bitmap_add(inImage, commandLine, inBitmapFilename)
		# resize bitmap area to just the specified length
		x, y = bitmapLayer.offsets
		if inOrient == ORNT_HORIZ:
			bmp_width = inLength
			bmp_height = bitmapLayer.height
		else:
			bmp_width = bitmapLayer.width
			bmp_height = inLength
		bitmapLayer.resize(bmp_width, bmp_height, x, y)
		slcd_common.set_user_item(slcd_common.windowed_bitmap(), inBitmapFilename)
	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return


#
# registrations
#
menu_path = slcd_common.menu_path()+"/Bi_tmap"

register(
	"python_fu_Reach_SLCD_Add_Bitmap",					# registration name
	"Add a bitmap to an existing screen image."+slcd_common.blurb_common(),	# blurb
	"Add a bitmap to an existing screen image.",		# help
	"Reach Technology, Inc.",							# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"June, 2013",										# date
	"Bi_tmap...",										# menu item in menu path below
	"*",												# image types
	[													# params
		# type			param name		visible name	default value
		(PF_IMAGE,		"image",		"Image",		None),
		(PF_FILENAME,	"bmp_file",		"bmp File",		slcd_common.get_user_closest(slcd_common.bitmap())),
#		(PF_TOGGLE,		"centered",		"Centered",		False),
		(PF_STRING,		"nickname",		"NickName",		"NickName"),
	],
	[],													# results
	Add_Bitmap,											# function name
	menu = menu_path									# menu path
)

register(
	"python_fu_Reach_SLCD_Add_Windowed_Bitmap",			# registration name
	"Add a windowed bitmap to an existing screen image."+slcd_common.blurb_common(),	# blurb
	"Add a windowed bitmap to an existing screen image.",								# help
	"Reach Technology, Inc.",							# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"June, 2013",										# date
	"_Windowed Bitmap...",								# menu item in menu path below
	"*",												# image types
	[													# params
		# type			param name		visible name	default value
		(PF_IMAGE,		"image",		"Image",		None),
		(PF_FILENAME,	"bmp_file",		"bmp File",		slcd_common.get_user_closest(slcd_common.windowed_bitmap())),
		(PF_RADIO,		"orient",		"Orient",				ORNT_HORIZ,
					(	# desc							value
						("Vertical",					ORNT_VERT),
						("Horizontal",					ORNT_HORIZ),
					)
		),
		(PF_INT,		"length",		"length",				1),
		(PF_INT,		"offset",		"offset",				0),
		(PF_STRING,		"nickname",		"NickName",		"NickName"),
	],
	[],													# results
	Add_Windowed_Bitmap,								# function name
	menu = menu_path									# menu path
)


main()
