#! /usr/bin/env python

#	Copyright 2013 by Reach Technology, Inc.
#	All Rights Reserved

from gimpfu import *
import slcd_common
	
ORNT_HORIZ = 1
ORNT_VERT = 0
INV_BOTTOM_RIGHT = 1
INV_TOP_LEFT = 0


#
# Adds a slider to an existing image.  Bitmap for slider is loaded from the
# specified file, into a new layer.  The SLCD command to draw the slider is
# used as the layer name, with the token COORDS used as a placeholder for
# the x/y coordinates.
#

def Add_Slider(
			inImage,
			inIndex,
			inBgBmpFile,
			inKnobBmpFile,
			inOrient,
			inInv,
			inHiVal,
			inLoVal,
			inNickName
):
	"""Add a slider to an existing screen image."""

	# sanity checks
	if not slcd_common.user_item_is_file(inBgBmpFile):
		fail("Invalid background bitmap file:\n" + inBgBmpFile)
	if not slcd_common.user_item_is_file(inKnobBmpFile):
		fail("Invalid knob bitmap file:\n" + inKnobBmpFile)
	if inHiVal == inLoVal:
		fail("Slider Hi Val and Lo Val are the same!")

	try:
		# SLIDER DEFINE syntax:
		#		sl idx bg x y slider off ornt inv cont hi lo
		#		"COORDS" is used for "x y", and will be replaced with actual x and y values later
		#		"KNOBOFF" is used for "offset", and will be replaced with actual offset value later
		bgBmp = slcd_common.get_bitmap_index(inBgBmpFile)
		knobBmp = slcd_common.get_bitmap_index(inKnobBmpFile)
		sliderCont = 1		# cont is always '1'
		sliderIndex = int(inIndex)
		commandLine =	"sl" + \
						" " + str(sliderIndex) + \
						" " + str(bgBmp) + \
						" " + slcd_common.coords() + \
						" " + str(knobBmp) + \
						" " + slcd_common.knob_offset() + \
						" " + str(inOrient) + \
						" " + str(inInv) + \
						" " + str(sliderCont) + \
						" " + str(inHiVal) + \
						" " + str(inLoVal) + \
						" " + slcd_common.make_nickname(inNickName)

		slcd_common.do_bitmap_add(inImage, commandLine, inBgBmpFile)
		slcd_common.do_bitmap_add(inImage, "// knob" + str(sliderIndex), inKnobBmpFile)
		slcd_common.set_user_item(slcd_common.slider_bg(), inBgBmpFile)
		slcd_common.set_user_item(slcd_common.slider_knob(), inKnobBmpFile)
	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return


#
# registrations
#
menu_path = slcd_common.menu_path()+"/_Slider"
blurb_index = "\n\nNote that the index value range overlaps the\nhotspot index value range."
slider_index_base = 128
slider_index_max = 255

register(
	"python_fu_Reach_SLCD_Add_Slider",							# registration name
	"Add a slider to an existing screen image."+slcd_common.blurb_common()+blurb_index,	# blurb
	"Add a slider to an existing screen image.",				# help
	"Reach Technology, Inc.",									# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"June, 2013",										# date
	"_Slider...",												# menu item in menu path below
	"*",														# image types
	[															# params
		# type			param name		visible name			default value
		(PF_IMAGE,		"image",		"Image",				None),
#		(PF_INT,		"index",		"Slider Index",			slcd_common.get_user_item(slcd_common.slider_index(), 128)),
		(PF_SPINNER,	"index",		"Slider Index",			slider_index_base, (slider_index_base, slider_index_max, 1)),
		(PF_FILENAME,	"bgBmpFile",	"Background Bitmap",	slcd_common.get_user_closest(slcd_common.slider_bg())),
		(PF_FILENAME,	"knobBmpFile",	"Knob Bitmap",			slcd_common.get_user_closest(slcd_common.slider_knob())),
		(PF_RADIO,		"orient",		"Orient",				ORNT_VERT,
					(	# desc							value
						("Horizontal",					ORNT_HORIZ),
						("Vertical",					ORNT_VERT),
					)
		),
		(PF_RADIO,		"invert",		"Invert",				INV_BOTTOM_RIGHT,
					(	# desc							value
						("Bottom/right is low",			INV_BOTTOM_RIGHT),
						("Top/left is low",				INV_TOP_LEFT),
					)
		),
		(PF_INT,		"hiVal",		"Hi Val",				1),
		(PF_INT,		"loVal",		"Lo Val",				1),
		(PF_STRING,		"nickname",		"NickName",				"NickName"),
	],
	[],															# results
	Add_Slider,													# function name
	menu = menu_path											# menu path
)


main()
