#! /usr/bin/env python

#	Copyright 2013 by Reach Technology, Inc.
#	All Rights Reserved

from gimpfu import *
import os, sys
import slcd_common
	

bitmap_cmnds	= ["xi", "xim", "xio"]
hotspot_cmnds	= ["x", "xs", "x", "xsnb", "xt", "xst", "xxy", "xxynb"]

# item type range base values - added to individual item numbers in .h file
ITEM_BUTTONS	= 0
ITEM_METERS		= 1000
ITEM_BITMAPS	= 2000
ITEM_SLIDERS	= 3000
ITEM_CHARTS		= 4000
ITEM_LEVELBARS	= 5000
ITEM_HOTSPOTS	= 6000

#
# Scan each layer other than the MainScreen layer, outputting the layer
# name to the file MACROS_TXT.  The token COORDS is replaced by the current
# coordinates of the bitmap in the layer.  The item type (button, meter,
# slirer, etc) item number and nickname are output to the file MACROS_H.
#

def OutputMacro(inImage, inMacrofileName):
	"""asdf"""
	# isolate the folder name, create include file name
	dirname = os.path.dirname(inMacrofileName)
	macro_core, macro_ext = os.path.splitext(inMacrofileName)
	include_name = macro_core + "_inc.h"
#	pdb.gimp_message("dirname: " + dirname)				# for debug
#	pdb.gimp_message("macro_core: " + macro_core)		# for debug
#	pdb.gimp_message("macro_ext: " + macro_ext)			# for debug
#	pdb.gimp_message("include_name: " + include_name)	# for debug

	# folder must exist already
	if os.path.exists(dirname):
		macro_fd = os.open(inMacrofileName, os.O_RDWR | os.O_CREAT | os.O_TRUNC)
		include_fd = os.open(include_name, os.O_RDWR | os.O_CREAT | os.O_TRUNC)

		# write header comment to macro file
		os.write(macro_fd, "# This file was automatically created by the Reach \"Output Macro\" utility.\n")
		os.write(macro_fd, "# Any changes made to this file manually will be lost if the utility is\n")
		os.write(macro_fd, "# used after making changes.\n")
		os.write(macro_fd, "# \n")
		os.write(macro_fd, "# The recommended approach is to copy the macro definition below into\n")
		os.write(macro_fd, "# your own macro file.\n")
		os.write(macro_fd, "# \n")
		os.write(macro_fd, "\n")

		# write header comment to incldue file
		os.write(include_fd, "// This file was automatically created by the Reach \"Output Macro\" utility.\n")
		os.write(include_fd, "\n")

		# start macro definition
		gimp.context_push()
		os.write(macro_fd, "#define slcd_setup_macro\n")
		layers = []
		current_active = inImage.active_layer
		bitmap_counter = 1

		for layer in inImage.layers:

			inImage.active_layer = layer
# don't autocrop - layer already sized to imported image
#			pdb.plug_in_autocrop_layer(inImage, layer)
			x, y = layer.offsets

			comment, token = layer.name.split(" ", 1)
			if comment == "//":
				# decode token
				if "knob" in token:
					knob_x = x
					knob_y = y
					knob_slider_id = token[4:]
				# add other token values here
#				else:
#					raise ValueError("unknown layer token: " + layer.name)
			else:
				# first split command name and item number from balance of command
				itemCmnd, itemNumber, junk = layer.name.split(" ", 2)
				# then split the command around the coords token
				part_1, part_2 = layer.name.split(slcd_common.coords(), 1)
				# and then get the nickanme
				junk, nickName = layer.name.split(slcd_common.nickname_token(), 1)
				if " #" in nickName:
					nickName = nickName.replace(" #", "_")

				# if layer is a slider, further split part_2 to replace the KNOBOFF token
				# with previously discovered knob_x value minus the slider x value
				if itemCmnd == "sl":
					if knob_slider_id == itemNumber:
						part2_a, part2_b = part_2.split(slcd_common.knob_offset(), 1)
						junk1, orient, junk2 = part2_b.split(" ", 2)
						if orient == "0":		# vertical slider - knob offset from background X
							if knob_x >= x:
								part_2 = part2_a + str(knob_x - x) + part2_b
							else:
								raise ValueError("invalid slider knob location for slider #: " + str(itemNumber))
								break
						elif orient == "1":		# horizoltal slider - knob offset from background Y
							if knob_y >= y:
								part_2 = part2_a + str(knob_y - y) + part2_b
							else:
								raise ValueError("invalid slider knob location for slider #: " + str(itemNumber))
								break
						else:
							raise ValueError("invalid slider knob oritntation: " + orient)
							break
					else:
						raise ValueError("slider/knob ID mismatch - slider_id: " + str(itemNumber) + "  knob: " + str(knob_slider_id))
						break

				# if layer is some sort of hotspot, need to adjust width (x1) and height (y1) values
				# by actual location.  x1 and y1 are the two parameters immediately after COORDS token.
				if itemCmnd in hotspot_cmnds:
					space, x1, y1, part2_tail = part_2.split(" ", 3)
					part_2 = " " + str(int(x1) + x) + " " + str(int(y1) + y) + " " + part2_tail
				
				# if layer is a meter, need to extract needle color from layer name,
				# then write commands to save current drawing state, set foreground color
				# to the specified needle color, draw meter ("final command"), then restore
				# drawing state (done later)
				if itemCmnd == "md":
					os.write(macro_fd, "ss\n")
					meter_1, meter_2 = part_2.split("NC:", 1)
					needleColorHex, junk = meter_2.split(" ", 1)
					os.write(macro_fd, "S " + needleColorHex + " 000000\n")	# background doesn't matter

				# write final command
				os.write(macro_fd, part_1)
				os.write(macro_fd, str(x) + " " + str(y))
				os.write(macro_fd, part_2 + "\n")
				
				# second part of meter command processing - restore drawing state
				if itemCmnd == "md":
					os.write(macro_fd, "sr\n")

				# create C-style definition for each item on screen
				# to associate item number with item's nick-name.
				itemOffset = 0
				if itemCmnd == "bd" or itemCmnd == "bdc":
					itemType = "BUTTON_"
#					itemOffset = ITEM_BUTTONS
				elif itemCmnd == "md":
					itemType = "METER_"
#					itemOffset = ITEM_METERS
				elif itemCmnd in bitmap_cmnds:
					itemType = "BITMAP_"
#					itemOffset = ITEM_BITMAPS
					# since bitmaps may be used multiple times, must create number manually
					itemNumber = str(bitmap_counter)
					bitmap_counter += 1
				elif itemCmnd == "sl":
					itemType = "SLIDER_"
#					itemOffset = ITEM_SLIDERS
				elif itemCmnd == "cd" or itemCmnd == "cdb":
					itemType = "CHART_"
#					itemOffset = ITEM_CHARTS
				elif itemCmnd == "ld":
					itemType = "LEVELBAR_"
#					itemOffset = ITEM_LEVELBARS
				elif itemCmnd in hotspot_cmnds:
					itemType = "HOTPOT_"
#					itemOffset = ITEM_HOTSPOTS
				else:
					raise ValueError("Unknown screen item:\n" + layer.name)
					break
				itemID = int(itemNumber) + itemOffset	# place itemNumber into approp. range
				os.write(include_fd, "#define " + itemType + nickName + " " + str(itemID) + "\n")

		# finish macro definition, close file, cleanup
		os.write(macro_fd, "#end\n")
		os.close(include_fd)

		os.close(macro_fd)

		inImage.active_layer = current_active
		gimp.displays_flush()
		gimp.context_pop()
	return


#
# registrations
#
menu_path = slcd_common.menu_path()

register(
	"python_fu_Reach_SLCD_Output_Macro",				# registration name
	"Outputs the screen setup macro for all defined items.",	# blurb
	"Outputs the screen setup macro for all defined items.",	# help
	"Reach Technology, Inc.",							# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"June, 2013",										# date
	"_Output Macro...",									# menu item in menu path below
	"*",												# image types
	[													# params
		# type			param name		visible name	default value
		(PF_IMAGE,		"image",		"Image",		None),
		(PF_FILENAME,	"macro_file",	"Macro File",	slcd_common.get_user_macrofile()),
	],
	[],													# results
	OutputMacro,										# function name
	menu = menu_path									# menu path
)

main()
