#! /usr/bin/env python

#	Copyright 2013 by Reach Technology, Inc.
#	All Rights Reserved

from gimpfu import *
import slcd_common
import os


#
# Creates an image sized to the selected screen type, or to the specified
# screen width and height.  Also handles landscape or portrait mode.
#

# standard screen descriptions and sizes
#	"4.3\" WQVGA (Landscape)			(480x272)"
#	"4.3\" WQVGA (Portrait)				(272x480)"
#	"5.7\" Standard QVGA (Landscape)	(320x240)"
#	"5.7\" Standard QVGA (Portrait)		(240x320)"
#	"5.7\" High Res VGA (Landscape)		(640x480)"
#	"7\" WVGA (Landscape)				(800x480)"
#	"8.4\" VGA (Landscape)				(640x480)"
#	"10.4\" SVGA (Landscape)			(800x600)"

# indicies into screenInfo array
SCR_TYPE_4_3_L =	0
SCR_TYPE_4_3_P =	1
SCR_TYPE_5_7S_L =	2
SCR_TYPE_5_7S_P =	3
SCR_TYPE_5_7H_L =	4
SCR_TYPE_7_L =		5
SCR_TYPE_8_4_L =	6
SCR_TYPE_10_4_L =	7

# indicies into a single screenInfo array entry
SCR_NAME =		0
SCR_WIDTH =		1
SCR_HEIGHT =	2

screenInfo =	[	# screen name with padding					width,	height
					["4.3\" WQVGA (Landscape)			",			480,	272],
					["4.3\" WQVGA (Portrait)				",		272,	480],
					["5.7\" Standard QVGA (Landscape)		",		320,	240],
					["5.7\" Standard QVGA (Portrait)		",		240,	320],
					["5.7\" High Res VGA (Landscape)		",		640,	480],
					["7\" WVGA (Landscape)				",			800,	480],
					["8.4\" VGA (Landscape)				",			640,	480],
					["10.4\" SVGA (Landscape)				",		800,	600]
				]

ORIENT_LANDSCAPE = 0
ORIENT_PORTRAIT = 1


#def queried():
#	"""testing"""
#	slcd_common.set_user_item(slcd_common.bitmap_folder(), "queried")
#	return
#
#
#def run():
#	"""testing"""
#	slcd_common.set_user_item(slcd_common.bitmap_folder(), "run")
#	return


def makeScreenString(index):
	"""Translate a screenType index into menu entry strings"""
	screenString = str(screenInfo[index][SCR_NAME]) + \
					"(" + \
					str(screenInfo[index][SCR_WIDTH]) + \
					"x" + \
					str(screenInfo[index][SCR_HEIGHT]) + \
					")"
	return screenString


def makeScreen (inScreenWidth, inScreenHeight, inOrient, inBitmapFolder):
	"""Create a new RGB image and layer based on the specified screen name and size"""

	# sanity checks
	if not slcd_common.user_item_is_dir(inBitmapFolder):
		fail("Invalid bitmap folder:\n" + inBitmapFolder)

	try:
		if inOrient == ORIENT_LANDSCAPE:
			width = inScreenWidth
			height = inScreenHeight
		else:
			height = inScreenWidth
			width = inScreenHeight
		image = gimp.Image(width, height, RGB)
		layer = gimp.Layer(image, slcd_common.mainscreen(), width, height, RGB_IMAGE, 100, NORMAL_MODE)
		image.add_layer(layer, 0)
		pdb.gimp_edit_fill(layer, BACKGROUND_FILL)
		gimp.Display(image)
		gimp.displays_flush()
		# remember bitmap folder
		slcd_common.set_user_item(slcd_common.bitmap_folder(), inBitmapFolder)
	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return


#def SLCD_Select(inScreenType, inOrient, inBitmapFolder):
def SLCD_Select(inScreenType, inBitmapFolder):
	"""Map a screenType into width and height, make the screen"""
	width = screenInfo[inScreenType][SCR_WIDTH]
	height = screenInfo[inScreenType][SCR_HEIGHT]
#	makeScreen(width, height, inOrient, inBitmapFolder)
	# NOTE: ORIENT_LANDSCAPE used since user selection now implies orientation
	makeScreen(width, height, ORIENT_LANDSCAPE, inBitmapFolder)
	return


def SLCD_Custom(inScreenWidth, inScreenHeight, inBitmapFolder):
	"""Make the screen with specified width and height"""
	# NOTE: ORIENT_LANDSCAPE used since user specified width/height directly
	makeScreen(inScreenWidth, inScreenHeight, ORIENT_LANDSCAPE, inBitmapFolder)
	return


#
# registrations
#
menu_path = slcd_common.menu_path()+"/_SLCD"

register(
	"python_fu_Reach_SLCD_Select",						# registration name
	"Create a screen image for a standard Reach display.",	# blurb
	"Create a screen image for a standard Reach display.",	# help
	"Reach Technology, Inc.",							# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"July, 2013",										# date
	"_Select...",										# menu item in menu path below
	"",													# image types
	[													# params
		# type			param name		visible name	default value
		(PF_RADIO,		"screenType",	"Type",			SCR_TYPE_4_3_L,
					(	# desc								value
						(makeScreenString(SCR_TYPE_4_3_L),	SCR_TYPE_4_3_L),
						(makeScreenString(SCR_TYPE_4_3_P),	SCR_TYPE_4_3_P),
						(makeScreenString(SCR_TYPE_5_7S_L),	SCR_TYPE_5_7S_L),
						(makeScreenString(SCR_TYPE_5_7S_P),	SCR_TYPE_5_7S_P),
						(makeScreenString(SCR_TYPE_5_7H_L),	SCR_TYPE_5_7H_L),
						(makeScreenString(SCR_TYPE_7_L),	SCR_TYPE_7_L),
						(makeScreenString(SCR_TYPE_8_4_L),	SCR_TYPE_8_4_L),
						(makeScreenString(SCR_TYPE_10_4_L),	SCR_TYPE_10_4_L)
					)
		),
# Display Modules using the SLCD5/SLCD5+ controllers don't support portrait mode
#		(PF_RADIO,		"orient",		"Orientation",	ORIENT_LANDSCAPE,
#					(	# desc								value
#						("Landscape",						ORIENT_LANDSCAPE),
#						("Portrait",						ORIENT_PORTRAIT)
#					)
#		),
		(PF_DIRNAME,	"bmp_folder",	"BMP Folder",	slcd_common.get_user_item(slcd_common.bitmap_folder(), os.getcwd())),
	],
	[],													# results
	SLCD_Select,										# function name
	menu = menu_path,									# menu path
# other parameters that gimpfu.py checks for:
#	domain = None,										# domain (language?) translation?
#	on_query = queried,									# called when script is loaded
#	on_run = run,										# called when script is run
)

register(
	"python_fu_Reach_SLCD_Custom",						# registration name
	"Create a custom size screen image for a Reach display.",	# blurb
	"Create a custom size screen image for a Reach display.",	# help
	"Reach Technology, Inc.",							# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"July, 2013",										# date
	"_Custom...",										# menu item in menu path below
	"",													# image types
	[													# params
		# type			param name		visible name	default value
		(PF_INT,		"width",		"Width",		800),
		(PF_INT,		"height",		"Height",		480),
# orientation is redundant in custom - just set width and height as needed
#		(PF_RADIO,		"orient",		"Orientation",	ORIENT_LANDSCAPE,
#					(	# desc								value
#						("Landscape",						ORIENT_LANDSCAPE),
#						("Portrait",						ORIENT_PORTRAIT)
#					)
#		),
		(PF_DIRNAME,	"bmp_folder",	"BMP Folder",	slcd_common.get_user_item(slcd_common.bitmap_folder(), os.getcwd())),
	],
	[],													# results
	SLCD_Custom,										# function name
	menu = menu_path									# menu path
)

main()
